"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useCheckedState = void 0;
var _react = require("react");
var _styleFunctions = require("../../theme/src/style-functions");
var _useDynamicStateStyle = require("./useDynamicStateStyle");
/**
 * Hook to manage a checked state and compose extra styling while a component is checked
 * @param props The props object of a component
 * @param styleFunctions List of style functions supported by the component. By default, boxStyleFunctions are used
 * @param activeComponentType Type of the component inside which the hook is used. By default, this value is set to 'basic'
 * @param animateable Whether the styles should be dynamically animated using Moti. By default, this value is set to 'true'
 * @param parentStateValue A override value to control the 'checked' state instead of the local state value
 * @returns The props object with updated styles according the current 'checked' state
 */
const useCheckedState = (props, styleFunctions = _styleFunctions.boxStyleFunctions, activeComponentType = "basic", animateable = true, parentStateValue = undefined) => {
  // Initialize the checked state to false
  const [checked, setChecked] = (0, _react.useState)(false);

  // Override the focused state if the parentStateValue is provided
  let finalCheckedFlag;
  if (parentStateValue !== undefined) {
    finalCheckedFlag = parentStateValue;
  } else {
    finalCheckedFlag = checked;
  }

  // Dynamically update the styles based on the checked state
  const propsWithCheckedStyles = (0, _useDynamicStateStyle.useDynamicStateStyle)(props, "_checked", finalCheckedFlag, styleFunctions, activeComponentType, animateable);

  // Return the checked state, a function to update the checked state, and the props with updated styles
  return {
    checked,
    setChecked,
    propsWithCheckedStyles
  };
};
exports.useCheckedState = useCheckedState;
//# sourceMappingURL=useCheckedState.js.map