import { useState } from "react";
import { boxStyleFunctions } from "../../theme/src/style-functions";
import { useDynamicStateStyle } from "./useDynamicStateStyle";

/**
 * Hook to manage a focused state and compose extra styling while a component is focused
 * @param props The props object of a component
 * @param styleFunctions List of style functions supported by the component. By default, boxStyleFunctions are used
 * @param activeComponentType Type of the component inside which the hook is used. By default, this value is set to 'basic'
 * @param animateable Whether the styles should be dynamically animated using Moti. By default, this value is set to 'true'
 * @param parentStateValue A override value to control the 'focused' state instead of the local state value
 * @returns The props object with updated styles according the current 'focused' state
 */
export const useFocusedState = (props, styleFunctions = boxStyleFunctions, activeComponentType = "basic", animateable = true, parentStateValue = undefined) => {
  // Initialize the focused state
  const [focused, setFocused] = useState(false);

  // Override the focused state if the parentStateValue is provided
  let finalFocusedFlag;
  if (parentStateValue !== undefined) {
    finalFocusedFlag = parentStateValue;
  } else {
    finalFocusedFlag = focused;
  }

  // Compose extra styling while a component is focused
  const propsWithFocusedStyles = useDynamicStateStyle(props, "_focused", finalFocusedFlag, styleFunctions, activeComponentType, animateable);

  // Return the props object with updated styles according the current 'focused' state
  return {
    focused,
    setFocused,
    propsWithFocusedStyles
  };
};
//# sourceMappingURL=useFocusedState.js.map